@@ -88,31 +88,5 @@ def run():
             os._exit(1)
 
 
-def plotly_serialization_helper(class_fullname):
-    name_parts = class_fullname.split(".")
-    module_name = ".".join(name_parts[:-1])
-    class_name = name_parts[-1]
-
-    module = importlib.import_module(module_name)
-    cls = getattr(module, class_name)
-
-    if not hasattr(cls, '__serialize__'):
-        #print(f"Registering {cls}")
-        def serialize(self, global_data):
-            #print("Serialising %s on downlink" % type(self))
-            return self.to_plotly_json()
-
-        @staticmethod
-        def new_deserialized(data, global_data):
-            #print("Deserialising %s on downlink" % cls)
-            return cls(data)
-
-        cls.__serialize__ = serialize
-        cls.__new_deserialized__ = new_deserialized
-        anvil.server.portable_class(cls, class_fullname)
-
-_server._serialization_helpers["plotly.graph_objs"] = plotly_serialization_helper
-
-
 if __name__ == "__main__":
     run()
